@@ -58,6 +58,7 @@ struct GsAppPrivate
 	gchar			*id;
 	gchar			*id_full;
 	gchar			*name;
+	GsAppQuality		 name_quality;
 	gchar			*icon;
 	GPtrArray		*sources;
 	GPtrArray		*source_ids;
@@ -65,8 +66,10 @@ struct GsAppPrivate
 	gchar			*version;
 	gchar			*version_ui;
 	gchar			*summary;
+	GsAppQuality		 summary_quality;
 	gchar			*summary_missing;
 	gchar			*description;
+	GsAppQuality		 description_quality;
 	GPtrArray		*screenshots;
 	GPtrArray		*categories;
 	GPtrArray		*keywords;
@@ -78,6 +81,7 @@ struct GsAppPrivate
 	gchar			*update_details;
 	gchar			*management_plugin;
 	gint			 rating;
+	gint			 rating_confidence;
 	GsAppRatingKind		 rating_kind;
 	guint64			 size;
 	GsAppKind		 kind;
@@ -153,6 +157,8 @@ gs_app_state_to_string (GsAppState state)
 		return "installed";
 	if (state == GS_APP_STATE_AVAILABLE)
 		return "available";
+	if (state == GS_APP_STATE_LOCAL)
+		return "local";
 	if (state == GS_APP_STATE_QUEUED)
 		return "queued";
 	if (state == GS_APP_STATE_INSTALLING)
@@ -261,6 +267,8 @@ gs_app_to_string (GsApp *app)
 		g_string_append_printf (str, "\tmenu-path:\t%s\n", priv->menu_path);
 	if (priv->rating != -1)
 		g_string_append_printf (str, "\trating:\t%i\n", priv->rating);
+	if (priv->rating_confidence != -1)
+		g_string_append_printf (str, "\trating-confidence:\t%i\n", priv->rating_confidence);
 	if (priv->rating_kind != GS_APP_RATING_KIND_UNKNOWN)
 		g_string_append_printf (str, "\trating-kind:\t%s\n",
 					priv->rating_kind == GS_APP_RATING_KIND_USER ?
@@ -289,66 +297,6 @@ gs_app_to_string (GsApp *app)
 	return g_string_free (str, FALSE);
 }
 
-/**
- * gs_app_subsume:
- *
- * Imports all the useful data from @other into @app.
- **/
-void
-gs_app_subsume (GsApp *app, GsApp *other)
-{
-	GList *keys;
-	GList *l;
-	GsAppPrivate *priv = app->priv;
-	GsAppPrivate *priv2 = other->priv;
-	const gchar *tmp;
-
-	/* wait for all the properties to be set */
-	g_object_freeze_notify (G_OBJECT (app));
-
-	/* an [updatable] installable package is more information than
-	 * just the fact that something is installed */
-	if (priv2->state == GS_APP_STATE_UPDATABLE &&
-	    priv->state == GS_APP_STATE_INSTALLED) {
-		/* we have to do the little dance to appease the
-		 * angry gnome controlling the state-machine */
-		gs_app_set_state (app, GS_APP_STATE_UNKNOWN);
-		gs_app_set_state (app, GS_APP_STATE_UPDATABLE);
-	}
-
-	/* save any properties we already know */
-	if (priv2->sources->len > 0)
-		gs_app_set_sources (app, priv2->sources);
-	if (priv2->project_group != NULL)
-		gs_app_set_project_group (app, priv2->project_group);
-	if (priv2->name != NULL)
-		gs_app_set_name (app, priv2->name);
-	if (priv2->summary != NULL)
-		gs_app_set_summary (app, priv2->summary);
-	if (priv2->description != NULL)
-		gs_app_set_description (app, priv2->description);
-	if (priv2->update_details != NULL)
-		gs_app_set_update_details (app, priv2->update_details);
-	if (priv2->update_version != NULL)
-		gs_app_set_update_version (app, priv2->update_version);
-	if (priv2->pixbuf != NULL)
-		gs_app_set_pixbuf (app, priv2->pixbuf);
-
-	/* also metadata */
-	keys = g_hash_table_get_keys (priv2->metadata);
-	for (l = keys; l != NULL; l = l->next) {
-		tmp = g_hash_table_lookup (priv->metadata, l->data);
-		if (tmp == NULL)
-			continue;
-		tmp = g_hash_table_lookup (priv2->metadata, l->data);
-		gs_app_set_metadata (other, l->data, tmp);
-	}
-	g_list_free (keys);
-
-	/* now emit all the changed signals */
-	g_object_thaw_notify (G_OBJECT (app));
-}
-
 /**
  * gs_app_get_id:
  **/
@@ -404,31 +352,16 @@ gs_app_get_state (GsApp *app)
 }
 
 /**
- * gs_app_set_state:
- *
- * This sets the state of the application. The following state diagram explains
- * the typical states. All applications start in state %GS_APP_STATE_UNKNOWN,
- * but the frontend is not supposed to see GsApps with this state, ever.
- * Backend plugins are reponsible for changing the state to one of the other
- * states before the GsApp is passed to the frontend. This is enforced by the
- * #GsPluginLoader.
- *
- * UPDATABLE --> INSTALLING --> INSTALLED
- * UPDATABLE --> REMOVING   --> AVAILABLE
- * INSTALLED --> REMOVING   --> AVAILABLE
- * AVAILABLE --> INSTALLING --> INSTALLED
- * AVAILABLE <--> QUEUED --> INSTALLING --> INSTALLED
- * UNKNOWN   --> UNAVAILABLE
+ * gs_app_set_state_internal:
  */
-void
-gs_app_set_state (GsApp *app, GsAppState state)
+static gboolean
+gs_app_set_state_internal (GsApp *app, GsAppState state)
 {
 	gboolean state_change_ok = FALSE;
 	GsAppPrivate *priv = app->priv;
 
-	g_return_if_fail (GS_IS_APP (app));
 	if (priv->state == state)
-		return;
+		return FALSE;
 
 	/* check the state change is allowed */
 	switch (priv->state) {
@@ -437,6 +370,7 @@ gs_app_set_state (GsApp *app, GsAppState state)
 		if (state == GS_APP_STATE_INSTALLED ||
 		    state == GS_APP_STATE_QUEUED ||
 		    state == GS_APP_STATE_AVAILABLE ||
+		    state == GS_APP_STATE_LOCAL ||
 		    state == GS_APP_STATE_UPDATABLE ||
 		    state == GS_APP_STATE_UNAVAILABLE)
 			state_change_ok = TRUE;
@@ -486,6 +420,12 @@ gs_app_set_state (GsApp *app, GsAppState state)
 		    state == GS_APP_STATE_AVAILABLE)
 			state_change_ok = TRUE;
 		break;
+	case GS_APP_STATE_LOCAL:
+		/* local has to go into an action state */
+		if (state == GS_APP_STATE_UNKNOWN ||
+		    state == GS_APP_STATE_INSTALLING)
+			state_change_ok = TRUE;
+		break;
 	default:
 		g_warning ("state %s unhandled",
 			   gs_app_state_to_string (priv->state));
@@ -498,16 +438,41 @@ gs_app_set_state (GsApp *app, GsAppState state)
 			   priv->id,
 			   gs_app_state_to_string (priv->state),
 			   gs_app_state_to_string (state));
-		return;
+		return FALSE;
 	}
 
 	priv->state = state;
 
 	if (state == GS_APP_STATE_UNKNOWN ||
-            state == GS_APP_STATE_AVAILABLE)
+	    state == GS_APP_STATE_LOCAL ||
+	    state == GS_APP_STATE_AVAILABLE)
 		app->priv->install_date = 0;
 
-	g_object_notify (G_OBJECT (app), "state");
+	return TRUE;
+}
+
+/**
+ * gs_app_set_state:
+ *
+ * This sets the state of the application. The following state diagram explains
+ * the typical states. All applications start in state %GS_APP_STATE_UNKNOWN,
+ * but the frontend is not supposed to see GsApps with this state, ever.
+ * Backend plugins are reponsible for changing the state to one of the other
+ * states before the GsApp is passed to the frontend. This is enforced by the
+ * #GsPluginLoader.
+ *
+ * UPDATABLE --> INSTALLING --> INSTALLED
+ * UPDATABLE --> REMOVING   --> AVAILABLE
+ * INSTALLED --> REMOVING   --> AVAILABLE
+ * AVAILABLE --> INSTALLING --> INSTALLED
+ * AVAILABLE <--> QUEUED --> INSTALLING --> INSTALLED
+ * UNKNOWN   --> UNAVAILABLE
+ */
+void
+gs_app_set_state (GsApp *app, GsAppState state)
+{
+	if (gs_app_set_state_internal (app, state))
+		g_object_notify (G_OBJECT (app), "state");
 }
 
 /**
@@ -616,12 +581,19 @@ gs_app_get_name (GsApp *app)
 /**
  * gs_app_set_name:
  * @app:	A #GsApp instance
+ * @quality:	A data quality, e.g. %GS_APP_QUALITY_LOWEST
  * @name:	The short localized name, e.g. "Calculator"
  */
 void
-gs_app_set_name (GsApp *app, const gchar *name)
+gs_app_set_name (GsApp *app, GsAppQuality quality, const gchar *name)
 {
 	g_return_if_fail (GS_IS_APP (app));
+
+	/* only save this if the data is sufficiently high quality */
+	if (quality < app->priv->name_quality)
+		return;
+	app->priv->name_quality = quality;
+
 	g_free (app->priv->name);
 	app->priv->name = g_strdup (name);
 }
@@ -644,6 +616,15 @@ gs_app_get_source_default (GsApp *app)
 void
 gs_app_add_source (GsApp *app, const gchar *source)
 {
+	const gchar *tmp;
+	guint i;
+
+	/* check source doesn't already exist */
+	for (i = 0; i < app->priv->sources->len; i++) {
+		tmp = g_ptr_array_index (app->priv->sources, i);
+		if (g_strcmp0 (tmp, source) == 0)
+			return;
+	}
 	g_ptr_array_add (app->priv->sources, g_strdup (source));
 }
 
@@ -1012,12 +993,19 @@ gs_app_get_summary (GsApp *app)
 /**
  * gs_app_set_summary:
  * @app:	A #GsApp instance
+ * @quality:	A data quality, e.g. %GS_APP_QUALITY_LOWEST
  * @summary:	The medium length localized name, e.g. "A graphical calculator for GNOME"
  */
 void
-gs_app_set_summary (GsApp *app, const gchar *summary)
+gs_app_set_summary (GsApp *app, GsAppQuality quality, const gchar *summary)
 {
 	g_return_if_fail (GS_IS_APP (app));
+
+	/* only save this if the data is sufficiently high quality */
+	if (quality < app->priv->summary_quality)
+		return;
+	app->priv->summary_quality = quality;
+
 	g_free (app->priv->summary);
 	app->priv->summary = g_strdup (summary);
 }
@@ -1035,12 +1023,19 @@ gs_app_get_description (GsApp *app)
 /**
  * gs_app_set_description:
  * @app:	A #GsApp instance
+ * @quality:	A data quality, e.g. %GS_APP_QUALITY_LOWEST
  * @summary:	The multiline localized description, e.g. "GNOME Calculator is a graphical calculator for GNOME....."
  */
 void
-gs_app_set_description (GsApp *app, const gchar *description)
+gs_app_set_description (GsApp *app, GsAppQuality quality, const gchar *description)
 {
 	g_return_if_fail (GS_IS_APP (app));
+
+	/* only save this if the data is sufficiently high quality */
+	if (quality < app->priv->summary_quality)
+		return;
+	app->priv->summary_quality = quality;
+
 	g_free (app->priv->description);
 	app->priv->description = g_strdup (description);
 }
@@ -1179,6 +1174,17 @@ gs_app_get_update_version_ui (GsApp *app)
 	return app->priv->update_version_ui;
 }
 
+/**
+ * gs_app_set_update_version_internal:
+ */
+static void
+gs_app_set_update_version_internal (GsApp *app, const gchar *update_version)
+{
+	g_free (app->priv->update_version);
+	app->priv->update_version = g_strdup (update_version);
+	gs_app_ui_versions_invalidate (app);
+}
+
 /**
  * gs_app_set_update_version:
  */
@@ -1186,9 +1192,7 @@ void
 gs_app_set_update_version (GsApp *app, const gchar *update_version)
 {
 	g_return_if_fail (GS_IS_APP (app));
-	g_free (app->priv->update_version);
-	app->priv->update_version = g_strdup (update_version);
-	gs_app_ui_versions_invalidate (app);
+	gs_app_set_update_version_internal (app, update_version);
 	g_object_notify (G_OBJECT (app), "version");
 }
 
@@ -1258,6 +1262,37 @@ gs_app_set_rating (GsApp *app, gint rating)
 	g_object_notify (G_OBJECT (app), "rating");
 }
 
+/**
+ * gs_app_get_rating_confidence:
+ * @app:	A #GsApp instance
+ *
+ * Return value: a predictor from 0 to 100, or -1 for unknown or invalid
+ */
+gint
+gs_app_get_rating_confidence (GsApp *app)
+{
+	g_return_val_if_fail (GS_IS_APP (app), -1);
+	return app->priv->rating_confidence;
+}
+
+/**
+ * gs_app_set_rating_confidence:
+ * @app:	A #GsApp instance
+ * @rating_confidence:	a predictor from 0 to 100, or -1 for unknown or invalid
+ *
+ * This is how confident the rating is statistically valid, expressed as a
+ * percentage.
+ * Applications with a high confidence typically have a large number of samples
+ * and can be trusted, but low confidence could mean that only one person has
+ * rated the application.
+ */
+void
+gs_app_set_rating_confidence (GsApp *app, gint rating_confidence)
+{
+	g_return_if_fail (GS_IS_APP (app));
+	app->priv->rating_confidence = rating_confidence;
+}
+
 /**
  * gs_app_get_rating_kind:
  */
@@ -1396,10 +1431,6 @@ gs_app_has_category (GsApp *app, const gchar *category)
 
 	g_return_val_if_fail (GS_IS_APP (app), FALSE);
 
-	/* nothing set */
-	if (app->priv->categories == NULL)
-		return FALSE;
-
 	/* find the category */
 	for (i = 0; i < app->priv->categories->len; i++) {
 		tmp = g_ptr_array_index (app->priv->categories, i);
@@ -1422,6 +1453,17 @@ gs_app_set_categories (GsApp *app, GPtrArray *categories)
 	app->priv->categories = g_ptr_array_ref (categories);
 }
 
+/**
+ * gs_app_add_category:
+ */
+void
+gs_app_add_category (GsApp *app, const gchar *category)
+{
+	g_return_if_fail (GS_IS_APP (app));
+	g_return_if_fail (category != NULL);
+	g_ptr_array_add (app->priv->categories, g_strdup (category));
+}
+
 /**
  * gs_app_get_keywords:
  */
@@ -1445,6 +1487,68 @@ gs_app_set_keywords (GsApp *app, GPtrArray *keywords)
 	app->priv->keywords = g_ptr_array_ref (keywords);
 }
 
+/**
+ * gs_app_subsume:
+ *
+ * Imports all the useful data from @other into @app.
+ *
+ * IMPORTANT: This method can be called from a thread as the notify signals
+ * are not sent.
+ **/
+void
+gs_app_subsume (GsApp *app, GsApp *other)
+{
+	const gchar *tmp;
+	GList *keys;
+	GList *l;
+	GsAppPrivate *priv2 = other->priv;
+	GsAppPrivate *priv = app->priv;
+	guint i;
+
+	/* an [updatable] installable package is more information than
+	 * just the fact that something is installed */
+	if (priv2->state == GS_APP_STATE_UPDATABLE &&
+	    priv->state == GS_APP_STATE_INSTALLED) {
+		/* we have to do the little dance to appease the
+		 * angry gnome controlling the state-machine */
+		gs_app_set_state_internal (app, GS_APP_STATE_UNKNOWN);
+		gs_app_set_state_internal (app, GS_APP_STATE_UPDATABLE);
+	}
+
+	/* save any properties we already know */
+	if (priv2->sources->len > 0)
+		gs_app_set_sources (app, priv2->sources);
+	if (priv2->project_group != NULL)
+		gs_app_set_project_group (app, priv2->project_group);
+	if (priv2->name != NULL)
+		gs_app_set_name (app, priv2->name_quality, priv2->name);
+	if (priv2->summary != NULL)
+		gs_app_set_summary (app, priv2->summary_quality, priv2->summary);
+	if (priv2->description != NULL)
+		gs_app_set_description (app, priv2->description_quality, priv2->description);
+	if (priv2->update_details != NULL)
+		gs_app_set_update_details (app, priv2->update_details);
+	if (priv2->update_version != NULL)
+		gs_app_set_update_version_internal (app, priv2->update_version);
+	if (priv2->pixbuf != NULL)
+		gs_app_set_pixbuf (app, priv2->pixbuf);
+	for (i = 0; i < priv2->categories->len; i++) {
+		tmp = g_ptr_array_index (priv2->categories, i);
+		gs_app_add_category (app, tmp);
+	}
+
+	/* also metadata */
+	keys = g_hash_table_get_keys (priv2->metadata);
+	for (l = keys; l != NULL; l = l->next) {
+		tmp = g_hash_table_lookup (priv->metadata, l->data);
+		if (tmp == NULL)
+			continue;
+		tmp = g_hash_table_lookup (priv2->metadata, l->data);
+		gs_app_set_metadata (other, l->data, tmp);
+	}
+	g_list_free (keys);
+}
+
 /**
  * gs_app_get_property:
  */
@@ -1501,16 +1605,22 @@ gs_app_set_property (GObject *object, guint prop_id, const GValue *value, GParam
 		gs_app_set_id (app, g_value_get_string (value));
 		break;
 	case PROP_NAME:
-		gs_app_set_name (app, g_value_get_string (value));
+		gs_app_set_name (app,
+				 GS_APP_QUALITY_UNKNOWN,
+				 g_value_get_string (value));
 		break;
 	case PROP_VERSION:
 		gs_app_set_version (app, g_value_get_string (value));
 		break;
 	case PROP_SUMMARY:
-		gs_app_set_summary (app, g_value_get_string (value));
+		gs_app_set_summary (app,
+				    GS_APP_QUALITY_UNKNOWN,
+				    g_value_get_string (value));
 		break;
 	case PROP_DESCRIPTION:
-		gs_app_set_description (app, g_value_get_string (value));
+		gs_app_set_description (app,
+					GS_APP_QUALITY_UNKNOWN,
+					g_value_get_string (value));
 		break;
 	case PROP_RATING:
 		gs_app_set_rating (app, g_value_get_int (value));
@@ -1519,7 +1629,7 @@ gs_app_set_property (GObject *object, guint prop_id, const GValue *value, GParam
 		gs_app_set_kind (app, g_value_get_uint (value));
 		break;
 	case PROP_STATE:
-		gs_app_set_state (app, g_value_get_uint (value));
+		gs_app_set_state_internal (app, g_value_get_uint (value));
 		break;
 	case PROP_INSTALL_DATE:
 		gs_app_set_install_date (app, g_value_get_uint64 (value));
@@ -1624,9 +1734,11 @@ gs_app_init (GsApp *app)
 {
 	app->priv = GS_APP_GET_PRIVATE (app);
 	app->priv->rating = -1;
+	app->priv->rating_confidence = -1;
 	app->priv->rating_kind = GS_APP_RATING_KIND_UNKNOWN;
 	app->priv->sources = g_ptr_array_new_with_free_func (g_free);
 	app->priv->source_ids = g_ptr_array_new_with_free_func (g_free);
+	app->priv->categories = g_ptr_array_new_with_free_func (g_free);
 	app->priv->related = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	app->priv->history = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	app->priv->screenshots = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
@@ -1677,8 +1789,7 @@ gs_app_finalize (GObject *object)
 		g_object_unref (priv->pixbuf);
 	if (priv->featured_pixbuf != NULL)
 		g_object_unref (priv->featured_pixbuf);
-	if (priv->categories != NULL)
-		g_ptr_array_unref (priv->categories);
+	g_ptr_array_unref (priv->categories);
 	if (priv->keywords != NULL)
 		g_ptr_array_unref (priv->keywords);
 
