@@ -38,6 +38,7 @@
 #include "gs-profile.h"
 #include "gs-shell-search-provider.h"
 #include "gs-offline-updates.h"
+#include "gs-folders.h"
 
 
 struct _GsApplication {
@@ -318,20 +319,6 @@ search_activated (GSimpleAction *action,
 	gs_shell_show_search (app->shell, search);
 }
 
-static void
-set_debug_level_activated (GSimpleAction *action,
-			   GVariant      *parameter,
-			   gpointer       data)
-{
-	if (g_variant_get_int32 (parameter) > 0) {
-		g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
-		g_debug ("enabled debugging");
-	} else {
-		g_debug ("enabled disabled");
-		g_setenv ("G_MESSAGES_DEBUG", "", TRUE);
-	}
-}
-
 static void
 details_activated (GSimpleAction *action,
 		   GVariant      *parameter,
@@ -354,6 +341,20 @@ details_activated (GSimpleAction *action,
 	gs_shell_show_search_result (app->shell, id, search);
 }
 
+static void
+filename_activated (GSimpleAction *action,
+		    GVariant      *parameter,
+		    gpointer       data)
+{
+	GsApplication *app = GS_APPLICATION (data);
+	const gchar *filename;
+
+	gs_application_initialize_ui (app);
+
+	g_variant_get (parameter, "(&s)", &filename);
+	gs_shell_show_filename (app->shell, filename);
+}
+
 static void
 launch_activated (GSimpleAction *action,
 		  GVariant      *parameter,
@@ -405,8 +406,8 @@ static GActionEntry actions[] = {
 	{ "profile", profile_activated, NULL, NULL, NULL },
 	{ "set-mode", set_mode_activated, "s", NULL, NULL },
 	{ "search", search_activated, "s", NULL, NULL },
-	{ "set-debug-level", set_debug_level_activated, "i", NULL, NULL },
 	{ "details", details_activated, "(ss)", NULL, NULL },
+	{ "filename", filename_activated, "(s)", NULL, NULL },
 	{ "launch", launch_activated, "s", NULL, NULL },
         { "clear-offline-updates", clear_offline_updates, NULL, NULL, NULL },
         { "show-offline-update-error", show_offline_updates_error, NULL, NULL, NULL },
@@ -429,6 +430,7 @@ gs_application_startup (GApplication *application)
 	gs_application_monitor_updates (GS_APPLICATION (application));
 	gs_application_provide_search (GS_APPLICATION (application));
 	gs_application_monitor_network (GS_APPLICATION (application));
+	gs_folders_convert ();
 }
 
 static void
@@ -458,12 +460,118 @@ gs_application_finalize (GObject *object)
 	G_OBJECT_CLASS (gs_application_parent_class)->finalize (object);
 }
 
+static gboolean
+gs_application_local_command_line (GApplication *app, gchar ***args, gint *status)
+{
+	GOptionContext *context;
+        gboolean gapplication_service = FALSE;
+	gchar *mode = NULL;
+	gchar *search = NULL;
+	gchar *local_filename = NULL;
+	gchar *id = NULL;
+	gboolean activate_ui = TRUE;
+	gboolean version = FALSE;
+	gboolean profile = FALSE;
+	gboolean verbose = FALSE;
+	gint argc;
+	const GOptionEntry options[] = {
+                { "gapplication-service", '\0', 0, G_OPTION_ARG_NONE, &gapplication_service,
+                   _("Enter GApplication service mode"), NULL }, 
+		{ "mode", '\0', 0, G_OPTION_ARG_STRING, &mode,
+		  /* TRANSLATORS: this is a command line option */
+		  _("Start up mode: either ‘updates’, ‘updated’, ‘installed’ or ‘overview’"), _("MODE") },
+		{ "search", '\0', 0, G_OPTION_ARG_STRING, &search,
+		  _("Search for applications"), _("SEARCH") },
+		{ "details", '\0', 0, G_OPTION_ARG_STRING, &id,
+		  _("Show application details"), _("ID") },
+		{ "local-filename", '\0', 0, G_OPTION_ARG_FILENAME, &local_filename,
+		  _("Show application details"), _("ID") },
+		{ "verbose", '\0', 0, G_OPTION_ARG_NONE, &verbose,
+		  _("Set the specified debugging level"), _("ID") },
+		{ "profile", 0, 0, G_OPTION_ARG_NONE, &profile,
+		  _("Show profiling information for the service"), NULL },
+		{ "version", 0, 0, G_OPTION_ARG_NONE, &version, NULL, NULL },
+		{ NULL}
+	};
+	GError *error = NULL;
+
+	context = g_option_context_new ("");
+	g_option_context_add_main_entries (context, options, NULL);
+
+	argc = g_strv_length (*args);
+	if (!g_option_context_parse (context, &argc, args, &error)) {
+		g_printerr ("%s\n", error->message);
+		g_error_free (error);
+		*status = 1;
+		goto out;
+	}
+
+	if (verbose)
+		g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+
+	if (version) {
+		g_print ("gnome-software " VERSION "\n");
+		*status = 0;
+		goto out;
+	}
+
+	if (gapplication_service) {
+		GApplicationFlags flags;
+
+		flags = g_application_get_flags (app);
+		g_application_set_flags (app, flags | G_APPLICATION_IS_SERVICE);
+		activate_ui = FALSE;
+	}
+
+	if (!g_application_register (app, NULL, &error)) {
+		g_printerr ("%s\n", error->message);
+		g_error_free (error);
+		*status = 1;
+		goto out;
+	}
+
+	if (profile) {
+		activate_ui = FALSE;
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"profile",
+						NULL);
+	}
+
+	if (mode != NULL) {
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"set-mode",
+						g_variant_new_string (mode));
+	} else if (search != NULL) {
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"search",
+						g_variant_new_string (search));
+	} else if (id != NULL) {
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"details",
+						g_variant_new ("(ss)", id, ""));
+	} else if (local_filename != NULL) {
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"filename",
+						g_variant_new ("(s)", local_filename));
+	} else if (activate_ui) {
+		g_application_activate (app);
+	}
+
+	*status = 0;
+
+out:
+	g_option_context_free (context);
+	g_free (local_filename);
+	return TRUE;
+}
+
 static void
 gs_application_class_init (GsApplicationClass *class)
 {
 	G_OBJECT_CLASS (class)->finalize = gs_application_finalize;
 	G_APPLICATION_CLASS (class)->startup = gs_application_startup;
 	G_APPLICATION_CLASS (class)->activate = gs_application_activate;
+	G_APPLICATION_CLASS (class)->local_command_line = gs_application_local_command_line;
 }
 
 GsApplication *
@@ -471,7 +579,6 @@ gs_application_new (void)
 {
 	return g_object_new (GS_APPLICATION_TYPE,
 			     "application-id", "org.gnome.Software",
-			     "flags", G_APPLICATION_IS_SERVICE,
 			     NULL);
 }
 
