@@ -113,6 +113,8 @@ gs_cmd_show_results_categories (GList *list)
 static GsPluginRefineFlags
 gs_cmd_refine_flag_from_string (const gchar *flag, GError **error)
 {
+	if (g_strcmp0 (flag, "all") == 0)
+		return 0xffff;
 	if (g_strcmp0 (flag, "licence") == 0)
 		return GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENCE;
 	if (g_strcmp0 (flag, "url") == 0)
@@ -277,6 +279,17 @@ main (int argc, char **argv)
 			if (!ret)
 				break;
 		}
+	} else if (argc == 3 && g_strcmp0 (argv[1], "filename-to-app") == 0) {
+		app = gs_plugin_loader_filename_to_app (plugin_loader,
+							argv[2],
+							refine_flags,
+							NULL,
+							&error);
+		if (app == NULL) {
+			ret = FALSE;
+		} else {
+			gs_plugin_add_app (&list, app);
+		}
 	} else if (argc == 2 && g_strcmp0 (argv[1], "updates") == 0) {
 		for (i = 0; i < repeat; i++) {
 			if (list != NULL)
@@ -303,6 +316,19 @@ main (int argc, char **argv)
 				break;
 			}
 		}
+	} else if (argc == 2 && g_strcmp0 (argv[1], "featured") == 0) {
+		for (i = 0; i < repeat; i++) {
+			if (list != NULL)
+				gs_plugin_list_free (list);
+			list = gs_plugin_loader_get_featured (plugin_loader,
+							      refine_flags,
+							      NULL,
+							      &error);
+			if (list == NULL) {
+				ret = FALSE;
+				break;
+			}
+		}
 	} else if (argc == 2 && g_strcmp0 (argv[1], "get-categories") == 0) {
 		for (i = 0; i < repeat; i++) {
 			if (list != NULL)
@@ -345,7 +371,7 @@ main (int argc, char **argv)
 				     GS_PLUGIN_ERROR_FAILED,
 				     "Did not recognise option, use 'installed', "
 				     "'updates', 'popular', 'get-categories', "
-				     "'get-category-apps', or 'search'");
+				     "'get-category-apps', 'filename-to-app', or 'search'");
 	}
 	if (!ret) {
 		g_print ("Failed: %s\n", error->message);
