@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -52,6 +52,7 @@ typedef struct {
 	GMainContext    *context;
 	GMainLoop	*loop;
 	gboolean	 ret;
+	GsApp		*app;
 } GsPluginLoaderHelper;
 
 static void
@@ -236,6 +237,51 @@ gs_plugin_loader_get_popular (GsPluginLoader *plugin_loader,
 	return helper.list;
 }
 
+static void
+gs_plugin_loader_get_featured_finish_sync (GsPluginLoader *plugin_loader,
+					  GAsyncResult *res,
+					  GsPluginLoaderHelper *helper)
+{
+	helper->list = gs_plugin_loader_get_featured_finish (plugin_loader,
+							     res,
+							     helper->error);
+	g_main_loop_quit (helper->loop);
+}
+
+/**
+ * gs_plugin_loader_get_featured:
+ **/
+GList *
+gs_plugin_loader_get_featured (GsPluginLoader *plugin_loader,
+			       GsPluginRefineFlags flags,
+			       GCancellable *cancellable,
+			       GError **error)
+{
+	GsPluginLoaderHelper helper;
+
+	/* create temp object */
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
+
+	/* run async method */
+	gs_plugin_loader_get_featured_async (plugin_loader,
+					     flags,
+					     cancellable,
+					     (GAsyncReadyCallback) gs_plugin_loader_get_featured_finish_sync,
+					     &helper);
+	g_main_loop_run (helper.loop);
+
+	g_main_context_pop_thread_default (helper.context);
+
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
+
+	return helper.list;
+}
+
 static void
 gs_plugin_loader_get_categories_finish_sync (GsPluginLoader *plugin_loader,
 					     GAsyncResult *res,
@@ -428,4 +474,104 @@ gs_plugin_loader_app_action (GsPluginLoader *plugin_loader,
 	return helper.ret;
 }
 
+/**
+ * gs_plugin_loader_refresh_finish_sync:
+ **/
+static void
+gs_plugin_loader_refresh_finish_sync (GsPluginLoader *plugin_loader,
+				      GAsyncResult *res,
+				      GsPluginLoaderHelper *helper)
+{
+	helper->ret = gs_plugin_loader_refresh_finish (plugin_loader,
+						       res,
+						       helper->error);
+	g_main_loop_quit (helper->loop);
+}
+
+/**
+ * gs_plugin_loader_refresh:
+ **/
+gboolean
+gs_plugin_loader_refresh (GsPluginLoader *plugin_loader,
+			  guint cache_age,
+			  GsPluginRefreshFlags flags,
+			  GCancellable *cancellable,
+			  GError **error)
+{
+	GsPluginLoaderHelper helper;
+
+	/* create temp object */
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
+
+	/* run async method */
+	gs_plugin_loader_refresh_async (plugin_loader,
+					cache_age,
+					flags,
+					cancellable,
+					(GAsyncReadyCallback) gs_plugin_loader_refresh_finish_sync,
+					&helper);
+	g_main_loop_run (helper.loop);
+
+	g_main_context_pop_thread_default (helper.context);
+
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
+
+	return helper.ret;
+}
+
+static void
+gs_plugin_loader_filename_to_app_finish_sync (GObject *source_object,
+					      GAsyncResult *res,
+					      gpointer user_data)
+{
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
+	GsPluginLoaderHelper *helper = (GsPluginLoaderHelper *) user_data;
+	helper->app = gs_plugin_loader_filename_to_app_finish (plugin_loader,
+							       res,
+							       helper->error);
+	g_main_loop_quit (helper->loop);
+}
+
+/**
+ * gs_plugin_loader_filename_to_app:
+ **/
+GsApp *
+gs_plugin_loader_filename_to_app (GsPluginLoader *plugin_loader,
+				  const gchar *filename,
+				  GsPluginRefineFlags flags,
+				  GCancellable *cancellable,
+				  GError **error)
+{
+	GsPluginLoaderHelper helper;
+
+	/* create temp object */
+	helper.app = NULL;
+	helper.context = g_main_context_new ();
+	helper.loop = g_main_loop_new (helper.context, FALSE);
+	helper.error = error;
+
+	g_main_context_push_thread_default (helper.context);
+
+	/* run async method */
+	gs_plugin_loader_filename_to_app_async (plugin_loader,
+						filename,
+						flags,
+						cancellable,
+						gs_plugin_loader_filename_to_app_finish_sync,
+						&helper);
+	g_main_loop_run (helper.loop);
+
+	g_main_context_pop_thread_default (helper.context);
+
+	g_main_loop_unref (helper.loop);
+	g_main_context_unref (helper.context);
+
+	return helper.app;
+}
+
 /* vim: set noexpandtab: */
