@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007-2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -45,6 +45,10 @@ GList		*gs_plugin_loader_get_popular		(GsPluginLoader	*plugin_loader,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
+GList		*gs_plugin_loader_get_featured		(GsPluginLoader	*plugin_loader,
+							 GsPluginRefineFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
 GList		*gs_plugin_loader_get_categories	(GsPluginLoader	*plugin_loader,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
@@ -64,11 +68,21 @@ gboolean	 gs_plugin_loader_app_action		(GsPluginLoader	*plugin_loader,
 							 GsPluginLoaderAction action,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_loader_refresh		(GsPluginLoader	*plugin_loader,
+							 guint		 cache_age,
+							 GsPluginRefreshFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
 GsApp		*gs_plugin_loader_get_app_by_id		(GsPluginLoader	*plugin_loader,
 							 const gchar	*id,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
+GsApp		*gs_plugin_loader_filename_to_app	(GsPluginLoader	*plugin_loader,
+							 const gchar	*filename,
+							 GsPluginRefineFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
 
 G_END_DECLS
 
