@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2007-2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2007-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -154,7 +154,7 @@ static gboolean
 gs_plugin_loader_run_refine_plugin (GsPluginLoader *plugin_loader,
 				    GsPlugin *plugin,
 				    const gchar *function_name_parent,
-				    GList *list,
+				    GList **list,
 				    GsPluginRefineFlags flags,
 				    GCancellable *cancellable,
 				    GError **error)
@@ -221,7 +221,7 @@ out:
 static gboolean
 gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 			     const gchar *function_name_parent,
-			     GList *list,
+			     GList **list,
 			     GsPluginRefineFlags flags,
 			     GCancellable *cancellable,
 			     GError **error)
@@ -230,9 +230,11 @@ gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 	GsPlugin *plugin;
 	gboolean ret = TRUE;
 	guint i;
+	GList *freeze_list;
 
 	/* freeze all apps */
-	for (l = list; l != NULL; l = l->next)
+	freeze_list = gs_plugin_list_copy (*list);
+	for (l = freeze_list; l != NULL; l = l->next)
 		g_object_freeze_notify (G_OBJECT (l->data));
 
 	/* run each plugin */
@@ -252,12 +254,13 @@ gs_plugin_loader_run_refine (GsPluginLoader *plugin_loader,
 	}
 
 	/* now emit all the changed signals */
-	for (l = list; l != NULL; l = l->next)
+	for (l = freeze_list; l != NULL; l = l->next)
 		g_object_thaw_notify (G_OBJECT (l->data));
 
 	/* dedupe applications we already know about */
-	gs_plugin_loader_list_dedupe (plugin_loader, list);
+	gs_plugin_loader_list_dedupe (plugin_loader, *list);
 out:
+	gs_plugin_list_free (freeze_list);
 	return ret;
 }
 
@@ -353,7 +356,7 @@ gs_plugin_loader_run_results (GsPluginLoader *plugin_loader,
 	/* run refine() on each one */
 	ret = gs_plugin_loader_run_refine (plugin_loader,
 					   function_name,
-					   list,
+					   &list,
 					   flags,
 					   cancellable,
 					   error);
@@ -438,8 +441,8 @@ gs_plugin_loader_app_is_valid (GsApp *app, gpointer user_data)
 
 	/* don't show unconverted packages in the application view */
 	if (gs_app_get_kind (app) == GS_APP_KIND_PACKAGE) {
-		g_debug ("app invalid as only a package %s",
-			 gs_plugin_loader_get_app_str (app));
+//		g_debug ("app invalid as only a package %s",
+//			 gs_plugin_loader_get_app_str (app));
 		return FALSE;
 	}
 
@@ -644,9 +647,10 @@ typedef struct {
 	GsPluginLoader			*plugin_loader;
 	GsPluginRefineFlags		 flags;
 	gchar				*value;
+	gchar				*filename;
+	guint				 cache_age;
 	GsCategory			*category;
 	GsApp				*app;
-	GsAppState			 state_progress;
 	GsAppState			 state_success;
 	GsAppState			 state_failure;
 } GsPluginLoaderAsyncState;
@@ -710,12 +714,20 @@ gs_plugin_loader_add_os_update_item (GList *list)
 	app_os = gs_app_new ("os-update.virtual");
 	gs_app_set_kind (app_os, GS_APP_KIND_OS_UPDATE);
 	gs_app_set_state (app_os, GS_APP_STATE_UPDATABLE);
-	/* TRANSLATORS: this is a group of updates that are not packages and
-	 * are ot shown in the main list */
-	gs_app_set_name (app_os, _("OS Updates"));
-	/* TRANSLATORS: this is a longer description of the os-update item */
-	gs_app_set_summary (app_os, _("Includes performance, stability and security improvements for all users."));
-	gs_app_set_description (app_os, _("Includes performance, stability and security improvements for all users."));
+	gs_app_set_name (app_os,
+			 GS_APP_QUALITY_NORMAL,
+			 /* TRANSLATORS: this is a group of updates that are not
+			  * packages and are not shown in the main list */
+			 _("OS Updates"));
+	gs_app_set_summary (app_os,
+			    GS_APP_QUALITY_NORMAL,
+			    /* TRANSLATORS: this is a longer description of the
+			     * os-update item */
+			    _("Includes performance, stability and security "
+			      "improvements for all users."));
+	gs_app_set_description (app_os,
+				GS_APP_QUALITY_NORMAL,
+				gs_app_get_summary (app_os));
 	for (l = list; l != NULL; l = l->next) {
 		app_tmp = GS_APP (l->data);
 		if (gs_app_get_kind (app_tmp) != GS_APP_KIND_PACKAGE)
@@ -912,6 +924,7 @@ gs_plugin_loader_get_installed_thread_cb (GSimpleAsyncResult *res,
 
 	/* filter package list */
 	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
+	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_non_system, NULL);
 	if (state->list == NULL) {
 		g_set_error_literal (&error,
 				     GS_PLUGIN_LOADER_ERROR,
@@ -1011,13 +1024,46 @@ gs_plugin_loader_get_installed_finish (GsPluginLoader *plugin_loader,
 
 /******************************************************************************/
 
+/**
+ * gs_plugin_loader_get_popular_filter_cb:
+ */
+static gboolean
+gs_plugin_loader_get_popular_filter_cb (GsApp *app, gpointer user_data)
+{
+	/* dont suggest already installed application */
+	if (gs_app_get_state (app) == GS_APP_STATE_INSTALLED)
+		return FALSE;
+
+	/* only show featured popular applications */
+	if (!gs_app_has_category (app, "featured"))
+		return FALSE;
+
+	/* require long description */
+	if (gs_app_get_description (app) == NULL)
+		return FALSE;
+
+	/* require at least one screenshot */
+	if (gs_app_get_screenshots (app)->len == 0)
+		return FALSE;
+
+	/* don't suggest applications we don't have a lot of confidence in,
+	 * just because one user rated the app 100% doesn't make it awesome
+	 * compared to an app with 300 votes of 85% */
+	if (gs_app_get_rating_confidence (app) < 90)
+		return FALSE;
+	if (gs_app_get_rating (app) < 75)
+		return FALSE;
+
+	return TRUE;
+}
+
 /**
  * gs_plugin_loader_get_popular_thread_cb:
  **/
 static void
 gs_plugin_loader_get_popular_thread_cb (GSimpleAsyncResult *res,
-					  GObject *object,
-					  GCancellable *cancellable)
+					GObject *object,
+					GCancellable *cancellable)
 {
 	GError *error = NULL;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
@@ -1037,6 +1083,9 @@ gs_plugin_loader_get_popular_thread_cb (GSimpleAsyncResult *res,
 
 	/* filter package list */
 	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
+	gs_plugin_list_filter (&state->list,
+			       gs_plugin_loader_get_popular_filter_cb,
+			       NULL);
 	if (state->list == NULL) {
 		g_set_error_literal (&error,
 				     GS_PLUGIN_LOADER_ERROR,
@@ -1047,6 +1096,9 @@ gs_plugin_loader_get_popular_thread_cb (GSimpleAsyncResult *res,
 		goto out;
 	}
 
+	/* shuffle around the list */
+	gs_plugin_list_randomize (&state->list);
+
 	/* success */
 	state->ret = TRUE;
 	gs_plugin_loader_get_all_state_finish (state, NULL);
@@ -1077,7 +1129,7 @@ gs_plugin_loader_get_popular_async (GsPluginLoader *plugin_loader,
 						user_data,
 						gs_plugin_loader_get_popular_async);
 	state->plugin_loader = g_object_ref (plugin_loader);
-	state->flags = flags;
+	state->flags = flags | GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING;
 	if (cancellable != NULL)
 		state->cancellable = g_object_ref (cancellable);
 
@@ -1144,6 +1196,7 @@ gs_plugin_loader_get_featured_thread_cb (GSimpleAsyncResult *res,
 
 	/* filter package list */
 	gs_plugin_list_filter (&state->list, gs_plugin_loader_app_is_valid, NULL);
+	gs_plugin_list_randomize (&state->list);
 	if (state->list == NULL) {
 		g_set_error_literal (&error,
 				     GS_PLUGIN_LOADER_ERROR,
@@ -1367,7 +1420,7 @@ gs_plugin_loader_search_thread_cb (GSimpleAsyncResult *res,
 	/* run refine() on each one */
 	ret = gs_plugin_loader_run_refine (plugin_loader,
 					   function_name,
-					   state->list,
+					   &state->list,
 					   state->flags,
 					   cancellable,
 					   &error);
@@ -1693,7 +1746,7 @@ gs_plugin_loader_get_category_apps_thread_cb (GSimpleAsyncResult *res,
 	/* run refine() on each one */
 	ret = gs_plugin_loader_run_refine (plugin_loader,
 					   function_name,
-					   state->list,
+					   &state->list,
 					   state->flags,
 					   cancellable,
 					   &error);
@@ -1854,7 +1907,7 @@ gs_plugin_loader_app_refine_thread_cb (GSimpleAsyncResult *res,
 	gs_plugin_add_app (&list, state->app);
 	state->ret = gs_plugin_loader_run_refine (plugin_loader,
 						  NULL,
-						  list,
+						  &list,
 						  state->flags,
 						  cancellable,
 						  &error);
@@ -1987,8 +2040,6 @@ gs_plugin_loader_app_action_thread_cb (GSimpleAsyncResult *res,
 	guint id;
 
 	/* add to list */
-	if (state->state_progress != GS_APP_STATE_UNKNOWN)
-		gs_app_set_state_in_idle (state->app, state->state_progress);
 	g_mutex_lock (&state->plugin_loader->priv->pending_apps_mutex);
 	g_ptr_array_add (state->plugin_loader->priv->pending_apps, g_object_ref (state->app));
 	g_mutex_unlock (&state->plugin_loader->priv->pending_apps_mutex);
@@ -2067,7 +2118,7 @@ load_install_queue (GsPluginLoader *plugin_loader, GError **error)
 	if (list != NULL) {
 		ret = gs_plugin_loader_run_refine (plugin_loader,
 						   NULL,
-						   list,
+						   &list,
 						   GS_PLUGIN_REFINE_FLAGS_DEFAULT,
 						   NULL, //FIXME?
 						   error);
@@ -2220,19 +2271,16 @@ gs_plugin_loader_app_action_async (GsPluginLoader *plugin_loader,
 	switch (action) {
 	case GS_PLUGIN_LOADER_ACTION_INSTALL:
 		state->function_name = "gs_plugin_app_install";
-		state->state_progress = GS_APP_STATE_INSTALLING;
 		state->state_success = GS_APP_STATE_INSTALLED;
 		state->state_failure = GS_APP_STATE_AVAILABLE;
 		break;
 	case GS_PLUGIN_LOADER_ACTION_REMOVE:
 		state->function_name = "gs_plugin_app_remove";
-		state->state_progress = GS_APP_STATE_REMOVING;
 		state->state_success = GS_APP_STATE_AVAILABLE;
 		state->state_failure = GS_APP_STATE_INSTALLED;
 		break;
 	case GS_PLUGIN_LOADER_ACTION_SET_RATING:
 		state->function_name = "gs_plugin_app_set_rating";
-		state->state_progress = GS_APP_STATE_UNKNOWN;
 		state->state_success = GS_APP_STATE_UNKNOWN;
 		state->state_failure = GS_APP_STATE_UNKNOWN;
 		break;
@@ -2408,7 +2456,7 @@ gs_plugin_loader_open_plugin (GsPluginLoader *plugin_loader,
 	gboolean ret;
 	GModule *module;
 	GsPluginGetNameFunc plugin_name = NULL;
-	GsPluginGetPriorityFunc plugin_prio = NULL;
+	GsPluginGetDepsFunc plugin_deps = NULL;
 	GsPlugin *plugin = NULL;
 
 	module = g_module_open (filename, 0);
@@ -2428,22 +2476,19 @@ gs_plugin_loader_open_plugin (GsPluginLoader *plugin_loader,
 		goto out;
 	}
 
-	/* get priority */
-	ret = g_module_symbol (module,
-			       "gs_plugin_get_priority",
-			       (gpointer *) &plugin_prio);
-	if (!ret) {
-		g_warning ("Plugin %s requires priority", filename);
-		g_module_close (module);
-		goto out;
-	}
+	/* get plugins this plugin depends on */
+	g_module_symbol (module,
+			 "gs_plugin_get_deps",
+			 (gpointer *) &plugin_deps);
 
 	/* print what we know */
 	plugin = g_slice_new0 (GsPlugin);
 	plugin->enabled = TRUE;
 	plugin->module = module;
 	plugin->pixbuf_size = 64;
-	plugin->priority = plugin_prio (plugin);
+	plugin->priority = 0.f;
+	plugin->deps = plugin_deps != NULL ? plugin_deps (plugin) : NULL;
+	plugin->settings = g_object_ref (plugin_loader->priv->settings);
 	plugin->name = g_strdup (plugin_name ());
 	plugin->status_update_fn = gs_plugin_loader_status_update_cb;
 	plugin->status_update_user_data = plugin_loader;
@@ -2496,6 +2541,25 @@ gs_plugin_loader_plugin_sort_fn (gconstpointer a, gconstpointer b)
 	return 0;
 }
 
+/**
+ * gs_plugin_loader_find_plugin:
+ */
+static GsPlugin *
+gs_plugin_loader_find_plugin (GsPluginLoader *plugin_loader,
+			      const gchar *plugin_name)
+{
+	GsPluginLoaderPrivate *priv = plugin_loader->priv;
+	GsPlugin *plugin;
+	guint i;
+
+	for (i = 0; i < priv->plugins->len; i++) {
+		plugin = g_ptr_array_index (priv->plugins, i);
+		if (g_strcmp0 (plugin->name, plugin_name) == 0)
+			return plugin;
+	}
+	return NULL;
+}
+
 /**
  * gs_plugin_loader_setup:
  */
@@ -2503,9 +2567,16 @@ gboolean
 gs_plugin_loader_setup (GsPluginLoader *plugin_loader, GError **error)
 {
 	const gchar *filename_tmp;
+	const gdouble dep_increment = 1.f;
+	gboolean changes;
 	gboolean ret = TRUE;
 	gchar *filename_plugin;
 	GDir *dir;
+	GsPlugin *dep;
+	GsPlugin *plugin;
+	guint dep_loop_check = 0;
+	guint i;
+	guint j;
 
 	g_return_val_if_fail (plugin_loader->priv->location != NULL, FALSE);
 
@@ -2532,6 +2603,46 @@ gs_plugin_loader_setup (GsPluginLoader *plugin_loader, GError **error)
 		g_free (filename_plugin);
 	} while (TRUE);
 
+	/* order by deps */
+	do {
+		changes = FALSE;
+		for (i = 0; i < plugin_loader->priv->plugins->len; i++) {
+			plugin = g_ptr_array_index (plugin_loader->priv->plugins, i);
+			if (plugin->deps == NULL)
+				continue;
+			for (j = 0; plugin->deps[j] != NULL && !changes; j++) {
+				dep = gs_plugin_loader_find_plugin (plugin_loader,
+								    plugin->deps[j]);
+				if (dep == NULL) {
+					g_warning ("cannot find plugin '%s'",
+						   plugin->deps[j]);
+					continue;
+				}
+				if (!dep->enabled)
+					continue;
+				if (plugin->priority <= dep->priority) {
+					g_debug ("%s [%.1f] requires %s [%.1f] "
+						 "so promoting to [%.1f]",
+						 plugin->name, plugin->priority,
+						 dep->name, dep->priority,
+						 dep->priority + dep_increment);
+					plugin->priority = dep->priority + dep_increment;
+					changes = TRUE;
+				}
+			}
+		}
+
+		/* check we're not stuck */
+		if (dep_loop_check++ > 100) {
+			ret = FALSE;
+			g_set_error (error,
+				     GS_PLUGIN_LOADER_ERROR,
+				     GS_PLUGIN_LOADER_ERROR_FAILED,
+				     "got stuck in dep loop");
+			goto out;
+		}
+	} while (changes);
+
 	/* sort by priority */
 	g_ptr_array_sort (plugin_loader->priv->plugins,
 			  gs_plugin_loader_plugin_sort_fn);
@@ -2579,6 +2690,7 @@ gs_plugin_loader_plugin_free (GsPlugin *plugin)
 	g_free (plugin->name);
 	g_object_unref (plugin->profile);
 	g_hash_table_unref (plugin->icon_cache);
+	g_object_unref (plugin->settings);
 	g_module_close (plugin->module);
 	g_slice_free (GsPlugin, plugin);
 }
@@ -2775,4 +2887,418 @@ gs_plugin_loader_set_network_status (GsPluginLoader *plugin_loader,
 	g_list_free_full (queue, g_object_unref);
 }
 
+/******************************************************************************/
+
+/**
+ * gs_plugin_loader_run_refresh_plugin:
+ **/
+static gboolean
+gs_plugin_loader_run_refresh_plugin (GsPluginLoader *plugin_loader,
+				     GsPlugin *plugin,
+				     guint cache_age,
+				     GsPluginRefreshFlags flags,
+				     GCancellable *cancellable,
+				     GError **error)
+{
+	const gchar *function_name = "gs_plugin_refresh";
+	gboolean exists;
+	gboolean ret = TRUE;
+	gchar *profile_id = NULL;
+	GError *error_local = NULL;
+	GsPluginRefreshFunc plugin_func = NULL;
+
+	exists = g_module_symbol (plugin->module,
+				  function_name,
+				  (gpointer *) &plugin_func);
+	if (!exists)
+		goto out;
+	profile_id = g_strdup_printf ("GsPlugin::%s(%s)",
+				      plugin->name, function_name);
+	gs_profile_start (plugin_loader->priv->profile, profile_id);
+	ret = plugin_func (plugin, cache_age, flags, cancellable, &error_local);
+	if (!ret) {
+		if (g_error_matches (error_local,
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED)) {
+			ret = TRUE;
+			g_debug ("not supported for plugin %s: %s",
+				 plugin->name,
+				 error_local->message);
+			g_clear_error (&error_local);
+		} else {
+			g_propagate_error (error, error_local);
+			goto out;
+		}
+	}
+out:
+	if (profile_id != NULL) {
+		gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_FINISHED);
+		gs_profile_stop (plugin_loader->priv->profile, profile_id);
+	}
+	g_free (profile_id);
+	return ret;
+}
+
+/**
+ * gs_plugin_loader_run_refresh:
+ **/
+static gboolean
+gs_plugin_loader_run_refresh (GsPluginLoader *plugin_loader,
+			      guint cache_age,
+			      GsPluginRefreshFlags flags,
+			      GCancellable *cancellable,
+			      GError **error)
+{
+	gboolean anything_ran = FALSE;
+	gboolean ret = TRUE;
+	GsPlugin *plugin;
+	guint i;
+
+	/* run each plugin */
+	for (i = 0; i < plugin_loader->priv->plugins->len; i++) {
+		plugin = g_ptr_array_index (plugin_loader->priv->plugins, i);
+		if (!plugin->enabled)
+			continue;
+		ret = g_cancellable_set_error_if_cancelled (cancellable, error);
+		if (ret) {
+			ret = FALSE;
+			goto out;
+		}
+		ret = gs_plugin_loader_run_refresh_plugin (plugin_loader,
+							   plugin,
+							   cache_age,
+							   flags,
+							   cancellable,
+							   error);
+		if (!ret)
+			goto out;
+		anything_ran = TRUE;
+	}
+
+	/* nothing ran */
+	if (!anything_ran) {
+		ret = FALSE;
+		g_set_error (error,
+			     GS_PLUGIN_LOADER_ERROR,
+			     GS_PLUGIN_LOADER_ERROR_FAILED,
+			     "no plugin could handle refresh");
+		goto out;
+	}
+out:
+	return ret;
+}
+
+/**
+ * gs_plugin_loader_refresh_state_finish:
+ **/
+static void
+gs_plugin_loader_refresh_state_finish (GsPluginLoaderAsyncState *state,
+				       const GError *error)
+{
+	if (state->ret) {
+		g_simple_async_result_set_op_res_gboolean (state->res, TRUE);
+	} else {
+		g_simple_async_result_set_from_error (state->res, error);
+	}
+
+	/* deallocate */
+	if (state->cancellable != NULL)
+		g_object_unref (state->cancellable);
+	g_object_unref (state->res);
+	g_object_unref (state->plugin_loader);
+	g_slice_free (GsPluginLoaderAsyncState, state);
+}
+
+/**
+ * gs_plugin_loader_refresh_thread_cb:
+ **/
+static void
+gs_plugin_loader_refresh_thread_cb (GSimpleAsyncResult *res,
+				    GObject *object,
+				    GCancellable *cancellable)
+{
+	GError *error = NULL;
+	GsPluginLoaderAsyncState *state = (GsPluginLoaderAsyncState *) g_object_get_data (G_OBJECT (cancellable), "state");
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
+
+	state->ret = gs_plugin_loader_run_refresh (plugin_loader,
+						   state->cache_age,
+						   state->flags,
+						   state->cancellable,
+						   &error);
+	if (!state->ret) {
+		gs_plugin_loader_refresh_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* success */
+	gs_plugin_loader_refresh_state_finish (state, NULL);
+out:
+	return;
+}
+
+/**
+ * gs_plugin_loader_refresh_async:
+ *
+ * This method calls all plugins that implement the gs_plugin_refine()
+ * function.
+ **/
+void
+gs_plugin_loader_refresh_async (GsPluginLoader *plugin_loader,
+				guint cache_age,
+				GsPluginRefreshFlags flags,
+				GCancellable *cancellable,
+				GAsyncReadyCallback callback,
+				gpointer user_data)
+{
+	GCancellable *tmp;
+	GsPluginLoaderAsyncState *state;
+
+	g_return_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	/* save state */
+	state = g_slice_new0 (GsPluginLoaderAsyncState);
+	state->res = g_simple_async_result_new (G_OBJECT (plugin_loader),
+						callback,
+						user_data,
+						gs_plugin_loader_refresh_async);
+	state->plugin_loader = g_object_ref (plugin_loader);
+	state->flags = flags;
+	state->cache_age = cache_age;
+	if (cancellable != NULL)
+		state->cancellable = g_object_ref (cancellable);
+
+	/* run in a thread */
+	tmp = g_cancellable_new ();
+	g_object_set_data (G_OBJECT (tmp), "state", state);
+	g_simple_async_result_run_in_thread (G_SIMPLE_ASYNC_RESULT (state->res),
+					     gs_plugin_loader_refresh_thread_cb,
+					     0,
+					     (GCancellable *) tmp);
+	g_object_unref (tmp);
+}
+
+/**
+ * gs_plugin_loader_refresh_finish:
+ *
+ * Return value: success
+ **/
+gboolean
+gs_plugin_loader_refresh_finish (GsPluginLoader *plugin_loader,
+				 GAsyncResult *res,
+				 GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_val_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader), FALSE);
+	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (res), FALSE);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+	/* failed */
+	simple = G_SIMPLE_ASYNC_RESULT (res);
+	if (g_simple_async_result_propagate_error (simple, error))
+		return FALSE;
+
+	/* grab detail */
+	return g_simple_async_result_get_op_res_gboolean (simple);
+}
+
+/******************************************************************************/
+
+/**
+ * gs_plugin_loader_filename_to_app_state_finish:
+ **/
+static void
+gs_plugin_loader_filename_to_app_state_finish (GsPluginLoaderAsyncState *state,
+					       const GError *error)
+{
+	if (state->app != NULL) {
+		g_simple_async_result_set_op_res_gpointer (state->res,
+							   g_object_ref (state->app),
+							   (GDestroyNotify) g_object_unref);
+	} else {
+		g_simple_async_result_set_from_error (state->res, error);
+	}
+
+	if (state->cancellable != NULL)
+		g_object_unref (state->cancellable);
+	if (state->app != NULL)
+		g_object_unref (state->app);
+	g_free (state->filename);
+	gs_plugin_list_free (state->list);
+	g_object_unref (state->res);
+	g_object_unref (state->plugin_loader);
+	g_slice_free (GsPluginLoaderAsyncState, state);
+}
+
+/**
+ * gs_plugin_loader_filename_to_app_thread_cb:
+ **/
+static void
+gs_plugin_loader_filename_to_app_thread_cb (GSimpleAsyncResult *res,
+					    GObject *object,
+					    GCancellable *cancellable)
+{
+	const gchar *function_name = "gs_plugin_filename_to_app";
+	gboolean ret = TRUE;
+	gchar *profile_id;
+	GError *error = NULL;
+	GsPluginLoaderAsyncState *state = (GsPluginLoaderAsyncState *) g_object_get_data (G_OBJECT (cancellable), "state");
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (object);
+	GsPlugin *plugin;
+	GsPluginFilenameToAppFunc plugin_func = NULL;
+	guint i;
+
+	/* run each plugin */
+	for (i = 0; i < plugin_loader->priv->plugins->len; i++) {
+		plugin = g_ptr_array_index (plugin_loader->priv->plugins, i);
+		if (!plugin->enabled)
+			continue;
+		ret = g_cancellable_set_error_if_cancelled (cancellable, &error);
+		if (ret) {
+			gs_plugin_loader_filename_to_app_state_finish (state, error);
+			g_error_free (error);
+			goto out;
+		}
+		ret = g_module_symbol (plugin->module,
+				       function_name,
+				       (gpointer *) &plugin_func);
+		if (!ret)
+			continue;
+		profile_id = g_strdup_printf ("GsPlugin::%s(%s)",
+					      plugin->name, function_name);
+		gs_profile_start (plugin_loader->priv->profile, profile_id);
+		ret = plugin_func (plugin, &state->list, state->filename, cancellable, &error);
+		if (!ret) {
+			gs_plugin_loader_filename_to_app_state_finish (state, error);
+			g_error_free (error);
+			goto out;
+		}
+		gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_FINISHED);
+		gs_profile_stop (plugin_loader->priv->profile, profile_id);
+		g_free (profile_id);
+	}
+
+	/* dedupe applications we already know about */
+	gs_plugin_loader_list_dedupe (plugin_loader, state->list);
+
+	/* run refine() on each one */
+	ret = gs_plugin_loader_run_refine (plugin_loader,
+					   function_name,
+					   &state->list,
+					   state->flags,
+					   cancellable,
+					   &error);
+	if (!ret) {
+		gs_plugin_loader_filename_to_app_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* filter package list */
+	gs_plugin_list_filter_duplicates (&state->list);
+	if (state->list == NULL) {
+		g_set_error (&error,
+			     GS_PLUGIN_LOADER_ERROR,
+			     GS_PLUGIN_LOADER_ERROR_NO_RESULTS,
+			     "no filename_to_app results to show");
+		gs_plugin_loader_filename_to_app_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* success */
+	if (g_list_length (state->list) != 1) {
+		g_set_error (&error,
+			     GS_PLUGIN_LOADER_ERROR,
+			     GS_PLUGIN_LOADER_ERROR_NO_RESULTS,
+			     "no application was created for %s",
+			     state->filename);
+		gs_plugin_loader_filename_to_app_state_finish (state, error);
+		g_error_free (error);
+		goto out;
+	}
+	state->app = g_object_ref (state->list->data);
+	gs_plugin_loader_filename_to_app_state_finish (state, NULL);
+out:
+	return;
+}
+
+/**
+ * gs_plugin_loader_filename_to_app_async:
+ *
+ * This method calls all plugins that implement the gs_plugin_add_filename_to_app()
+ * function. The plugins can either return #GsApp objects of kind
+ * %GS_APP_KIND_NORMAL for bonafide applications, or #GsApp's of kind
+ * %GS_APP_KIND_PACKAGE for packages that may or may not be applications.
+ *
+ * Once the list of updates is refined, some of the #GsApp's of kind
+ * %GS_APP_KIND_PACKAGE will have been promoted to a kind of %GS_APP_KIND_NORMAL,
+ * or if they are core applications.
+ **/
+void
+gs_plugin_loader_filename_to_app_async (GsPluginLoader *plugin_loader,
+					const gchar *filename,
+					GsPluginRefineFlags flags,
+					GCancellable *cancellable,
+					GAsyncReadyCallback callback,
+					gpointer user_data)
+{
+	GCancellable *tmp;
+	GsPluginLoaderAsyncState *state;
+
+	g_return_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+	/* save state */
+	state = g_slice_new0 (GsPluginLoaderAsyncState);
+	state->res = g_simple_async_result_new (G_OBJECT (plugin_loader),
+						callback,
+						user_data,
+						gs_plugin_loader_filename_to_app_async);
+	state->plugin_loader = g_object_ref (plugin_loader);
+	state->flags = flags;
+	state->filename = g_strdup (filename);
+	if (cancellable != NULL)
+		state->cancellable = g_object_ref (cancellable);
+
+	/* run in a thread */
+	tmp = g_cancellable_new ();
+	g_object_set_data (G_OBJECT (tmp), "state", state);
+	g_simple_async_result_run_in_thread (G_SIMPLE_ASYNC_RESULT (state->res),
+					     gs_plugin_loader_filename_to_app_thread_cb,
+					     0,
+					     (GCancellable *) tmp);
+	g_object_unref (tmp);
+}
+
+/**
+ * gs_plugin_loader_filename_to_app_finish:
+ *
+ * Return value: (element-type GsApp) (transfer full): An application, or %NULL
+ **/
+GsApp *
+gs_plugin_loader_filename_to_app_finish (GsPluginLoader *plugin_loader,
+					 GAsyncResult *res,
+					 GError **error)
+{
+	GSimpleAsyncResult *simple;
+
+	g_return_val_if_fail (GS_IS_PLUGIN_LOADER (plugin_loader), NULL);
+	g_return_val_if_fail (G_IS_SIMPLE_ASYNC_RESULT (res), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* failed */
+	simple = G_SIMPLE_ASYNC_RESULT (res);
+	if (g_simple_async_result_propagate_error (simple, error))
+		return NULL;
+
+	/* grab application */
+	return g_object_ref (g_simple_async_result_get_op_res_gpointer (simple));
+}
+
+/******************************************************************************/
+
 /* vim: set noexpandtab: */
