@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012-2014 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -61,6 +61,7 @@ typedef gboolean (*GsPluginListFilter)	(GsApp		*app,
 struct GsPlugin {
 	GModule			*module;
 	gdouble			 priority;	/* largest number gets run first */
+	const gchar		**deps;		/* allow-none */
 	gboolean		 enabled;
 	gchar			*name;
 	GsPluginPrivate		*priv;
@@ -71,6 +72,7 @@ struct GsPlugin {
 	gpointer		 updates_changed_user_data;
 	GsProfile		*profile;
 	GHashTable		*icon_cache;	/* key is the id.desktop */
+	GSettings		*settings;
 };
 
 typedef enum {
@@ -94,13 +96,18 @@ typedef enum {
 	GS_PLUGIN_REFINE_FLAGS_LAST
 } GsPluginRefineFlags;
 
+typedef enum {
+	GS_PLUGIN_REFRESH_FLAGS_UPDATES			= 1 << 0,
+	GS_PLUGIN_REFRESH_FLAGS_LAST
+} GsPluginRefreshFlags;
+
 /* helpers */
 #define	GS_PLUGIN_ERROR					1
 #define	GS_PLUGIN_GET_PRIVATE(x)			g_new0 (x,1)
 #define	GS_PLUGIN(x)					((GsPlugin *) x);
 
 typedef const gchar	*(*GsPluginGetNameFunc)		(void);
-typedef gdouble		 (*GsPluginGetPriorityFunc)	(GsPlugin	*plugin);
+typedef const gchar	**(*GsPluginGetDepsFunc)	(GsPlugin	*plugin);
 typedef void		 (*GsPluginFunc)		(GsPlugin	*plugin);
 typedef gboolean	 (*GsPluginSearchFunc)		(GsPlugin	*plugin,
 							 gchar		**value,
@@ -121,16 +128,28 @@ typedef gboolean	 (*GsPluginActionFunc)		(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 typedef gboolean	 (*GsPluginRefineFunc)		(GsPlugin	*plugin,
-							 GList		*list,
+							 GList		**list,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
+typedef gboolean	 (*GsPluginRefreshFunc	)	(GsPlugin	*plugin,
+							 guint		 cache_age,
+							 GsPluginRefreshFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
+typedef gboolean	 (*GsPluginFilenameToAppFunc)	(GsPlugin	*plugin,
+							 GList		**list,
+							 const gchar	*filename,
+							 GCancellable	*cancellable,
+							 GError		**error);
 
 const gchar	*gs_plugin_get_name			(void);
 void		 gs_plugin_initialize			(GsPlugin	*plugin);
 void		 gs_plugin_destroy			(GsPlugin	*plugin);
 void		 gs_plugin_set_enabled			(GsPlugin	*plugin,
 							 gboolean	 enabled);
+gboolean	 gs_plugin_check_distro_id		(GsPlugin	*plugin,
+							 const gchar	*distro_id);
 void		 gs_plugin_add_app			(GList		**list,
 							 GsApp		*app);
 void		 gs_plugin_list_free			(GList		*list);
@@ -139,6 +158,7 @@ void		 gs_plugin_list_filter			(GList		**list,
 							 GsPluginListFilter func,
 							 gpointer	 user_data);
 void		 gs_plugin_list_filter_duplicates	(GList		**list);
+void		 gs_plugin_list_randomize		(GList		**list);
 
 void		 gs_plugin_status_update		(GsPlugin	*plugin,
 							 GsApp		*app,
@@ -150,7 +170,7 @@ gboolean	 gs_plugin_add_search			(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
 							 GError		**error);
-gdouble		 gs_plugin_get_priority			(GsPlugin	*plugin);
+const gchar	**gs_plugin_get_deps			(GsPlugin	*plugin);
 gboolean	 gs_plugin_add_installed		(GsPlugin	*plugin,
 							 GList		**list,
 							 GCancellable	*cancellable,
@@ -181,7 +201,7 @@ gboolean	 gs_plugin_add_featured			(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 gboolean	 gs_plugin_refine			(GsPlugin	*plugin,
-							 GList		*list,
+							 GList		**list,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
@@ -201,6 +221,16 @@ gboolean	 gs_plugin_app_set_rating		(GsPlugin	*plugin,
 							 GsApp		*app,
 							 GCancellable	*cancellable,
 							 GError		**error);
+gboolean	 gs_plugin_refresh			(GsPlugin	*plugin,
+							 guint		 cache_age,
+							 GsPluginRefreshFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
+gboolean	 gs_plugin_filename_to_app		(GsPlugin	*plugin,
+							 GList		**list,
+							 const gchar	*filename,
+							 GCancellable	*cancellable,
+							 GError		**error);
 
 G_END_DECLS
 
