@@ -36,6 +36,11 @@ static void	gs_shell_details_finalize	(GObject	*object);
 
 #define GS_SHELL_DETAILS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_SHELL_DETAILS, GsShellDetailsPrivate))
 
+typedef enum {
+	GS_SHELL_DETAILS_STATE_LOADING,
+	GS_SHELL_DETAILS_STATE_READY
+} GsShellDetailsState;
+
 struct GsShellDetailsPrivate
 {
 	GsPluginLoader		*plugin_loader;
@@ -62,6 +67,45 @@ gs_shell_details_invalidate (GsShellDetails *shell_details)
 	shell_details->priv->cache_valid = FALSE;
 }
 
+/**
+ * gs_shell_details_set_state:
+ **/
+static void
+gs_shell_details_set_state (GsShellDetails *shell_details,
+			    GsShellDetailsState state)
+{
+	GsShellDetailsPrivate *priv = shell_details->priv;
+	GtkWidget *widget;
+
+	/* spinner */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "spinner_details"));
+	switch (state) {
+	case GS_SHELL_DETAILS_STATE_LOADING:
+		gtk_spinner_start (GTK_SPINNER (widget));
+		gtk_widget_show (widget);
+		break;
+	case GS_SHELL_DETAILS_STATE_READY:
+		gtk_spinner_stop (GTK_SPINNER (widget));
+		gtk_widget_hide (widget);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	/* stack */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "stack_details"));
+	switch (state) {
+	case GS_SHELL_DETAILS_STATE_LOADING:
+		gtk_stack_set_visible_child_name (GTK_STACK (widget), "spinner");
+		break;
+	case GS_SHELL_DETAILS_STATE_READY:
+		gtk_stack_set_visible_child_name (GTK_STACK (widget), "ready");
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 /**
  * gs_shell_details_refresh:
  **/
@@ -101,6 +145,7 @@ gs_shell_details_refresh (GsShellDetails *shell_details)
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_install"));
 	switch (state) {
 	case GS_APP_STATE_AVAILABLE:
+	case GS_APP_STATE_LOCAL:
 		gtk_widget_set_visible (widget, TRUE);
 		gtk_widget_set_sensitive (widget, TRUE);
 		gtk_style_context_add_class (gtk_widget_get_style_context (widget), "suggested-action");
@@ -159,6 +204,7 @@ gs_shell_details_refresh (GsShellDetails *shell_details)
 			gtk_style_context_remove_class (gtk_widget_get_style_context (widget), "destructive-action");
 			gtk_button_set_label (GTK_BUTTON (widget), _("_Cancel"));
 			break;
+		case GS_APP_STATE_LOCAL:
 		case GS_APP_STATE_AVAILABLE:
 		case GS_APP_STATE_INSTALLING:
 		case GS_APP_STATE_UNAVAILABLE:
@@ -173,7 +219,7 @@ gs_shell_details_refresh (GsShellDetails *shell_details)
 	}
 
 	/* spinner */
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_spinner"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_spinner_end"));
 	if (kind == GS_APP_KIND_SYSTEM) {
 		gtk_widget_set_visible (widget, FALSE);
 		gtk_spinner_stop (GTK_SPINNER (widget));
@@ -185,6 +231,7 @@ gs_shell_details_refresh (GsShellDetails *shell_details)
 		case GS_APP_STATE_QUEUED:
 		case GS_APP_STATE_UPDATABLE:
 		case GS_APP_STATE_UNAVAILABLE:
+		case GS_APP_STATE_LOCAL:
 			gtk_widget_set_visible (widget, FALSE);
 			gtk_spinner_stop (GTK_SPINNER (widget));
 			break;
@@ -476,6 +523,18 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 	tmp = gs_app_get_description (priv->app);
 	gs_shell_details_set_description (shell_details, tmp);
 
+	/* do not show the 'Details' header if we are a local file and have no
+	 * long description */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "application_details_details_title"));
+	switch (gs_app_get_state (priv->app)) {
+	case GS_APP_STATE_LOCAL:
+		gtk_widget_set_visible (widget, tmp != NULL);
+		break;
+	default:
+		gtk_widget_set_visible (widget, TRUE);
+		break;
+	}
+
 	/* set the icon */
 	tmp = gs_app_get_metadata_item (priv->app, "DataDir::desktop-icon");
 	if (tmp != NULL) {
@@ -488,6 +547,8 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 	}
 	if (pixbuf == NULL)
 		pixbuf = gs_app_get_pixbuf (priv->app);
+	if( pixbuf == NULL && gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL)
+		pixbuf = gs_pixbuf_load ("package-x-generic-symbolic", 96, NULL);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "application_details_icon"));
 	if (pixbuf != NULL) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (widget), pixbuf);
@@ -602,6 +663,11 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		break;
 	}
 
+	/* don't show a missing rating on a local file */
+	if (gs_app_get_state (priv->app) == GS_APP_STATE_LOCAL &&
+	    gs_app_get_rating (priv->app) < 0)
+		gtk_widget_set_visible (priv->star, FALSE);
+
 	/* only mark the stars as sensitive if the application is installed */
 	gtk_widget_set_sensitive (priv->star,
 				  gs_app_get_state (priv->app) == GS_APP_STATE_INSTALLED);
@@ -623,6 +689,17 @@ gs_shell_details_refresh_all (GsShellDetails *shell_details)
 		break;
 	}
 
+	/* only show support box for non-local packages */
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "box_details_support"));
+	switch (gs_app_get_state (priv->app)) {
+	case GS_APP_STATE_LOCAL:
+		gtk_widget_set_visible (widget, FALSE);
+		break;
+	default:
+		gtk_widget_set_visible (widget, TRUE);
+		break;
+	}
+
 	/* make history button insensitive if there is none */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_history"));
 	switch (gs_app_get_id_kind (priv->app)) {
@@ -662,6 +739,72 @@ gs_shell_details_app_refine_cb (GObject *source,
 		return;
 	}
 	gs_shell_details_refresh_all (shell_details);
+	gs_shell_details_set_state (shell_details, GS_SHELL_DETAILS_STATE_READY);
+}
+
+/**
+ * gs_shell_details_filename_to_app_cb:
+ **/
+static void
+gs_shell_details_filename_to_app_cb (GObject *source,
+				     GAsyncResult *res,
+				     gpointer user_data)
+{
+	gchar *tmp;
+	GError *error = NULL;
+	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source);
+	GsShellDetails *shell_details = GS_SHELL_DETAILS (user_data);
+	GsShellDetailsPrivate *priv = shell_details->priv;
+
+	if (priv->app != NULL)
+		g_object_unref (priv->app);
+	priv->app = gs_plugin_loader_filename_to_app_finish(plugin_loader,
+							    res,
+							    &error);
+	if (priv->app == NULL) {
+		g_warning ("failed to convert to GsApp: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	/* save app */
+	g_signal_connect_object (priv->app, "notify::state",
+				 G_CALLBACK (gs_shell_details_notify_state_changed_cb),
+				 shell_details, 0);
+	g_signal_connect_object (priv->app, "notify::size",
+				 G_CALLBACK (gs_shell_details_notify_state_changed_cb),
+				 shell_details, 0);
+	g_signal_connect_object (priv->app, "notify::licence",
+				 G_CALLBACK (gs_shell_details_notify_state_changed_cb),
+				 shell_details, 0);
+
+	/* print what we've got */
+	tmp = gs_app_to_string (priv->app);
+	g_debug ("%s", tmp);
+	g_free (tmp);
+
+	/* change widgets */
+	gs_shell_details_refresh (shell_details);
+	gs_shell_details_refresh_screenshots (shell_details);
+	gs_shell_details_refresh_all (shell_details);
+	gs_shell_details_set_state (shell_details, GS_SHELL_DETAILS_STATE_READY);
+}
+
+/**
+ * gs_shell_details_set_filename:
+ **/
+void
+gs_shell_details_set_filename (GsShellDetails *shell_details, const gchar *filename)
+{
+	GsShellDetailsPrivate *priv = shell_details->priv;
+	gs_shell_details_set_state (shell_details, GS_SHELL_DETAILS_STATE_LOADING);
+	gs_plugin_loader_filename_to_app_async (priv->plugin_loader,
+						filename,
+						GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+						GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING,
+						priv->cancellable,
+						gs_shell_details_filename_to_app_cb,
+						shell_details);
 }
 
 /**
@@ -679,6 +822,7 @@ gs_shell_details_set_app (GsShellDetails *shell_details, GsApp *app)
 	g_free (app_dump);
 
 	/* get extra details about the app */
+	gs_shell_details_set_state (shell_details, GS_SHELL_DETAILS_STATE_LOADING);
 	gs_plugin_loader_app_refine_async (priv->plugin_loader, app,
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENCE |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE |
