@@ -126,7 +126,10 @@ gs_shell_change_mode (GsShell *shell,
 	 */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_update_all"));
 	gtk_widget_hide (widget);
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_spinner"));
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_spinner_start"));
+	gtk_spinner_stop (GTK_SPINNER (widget));
+	gtk_widget_hide (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_spinner_end"));
 	gtk_spinner_stop (GTK_SPINNER (widget));
 	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header_label"));
@@ -137,6 +140,8 @@ gs_shell_change_mode (GsShell *shell,
 	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_select"));
 	gtk_widget_hide (widget);
+	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_refresh"));
+	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "application_details_header"));
 	gtk_widget_hide (widget);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_back"));
@@ -192,8 +197,12 @@ gs_shell_change_mode (GsShell *shell,
 		gs_shell_updates_refresh (priv->shell_updates, TRUE, scroll_up);
 		break;
 	case GS_SHELL_MODE_DETAILS:
-		gs_shell_details_set_app (priv->shell_details, app);
-		gs_shell_details_refresh (priv->shell_details);
+		if (app != NULL) {
+			gs_shell_details_set_app (priv->shell_details, app);
+			gs_shell_details_refresh (priv->shell_details);
+		}
+		if (data != NULL)
+			gs_shell_details_set_filename (priv->shell_details, data);
 		break;
 	case GS_SHELL_MODE_CATEGORY:
 		gs_shell_category_set_category (priv->shell_category,
@@ -595,6 +604,7 @@ gs_shell_show_app (GsShell *shell, GsApp *app)
 {
 	save_back_entry (shell);
 	gs_shell_change_mode (shell, GS_SHELL_MODE_DETAILS, app, NULL, TRUE);
+	gs_shell_activate (shell);
 }
 
 void
@@ -615,6 +625,14 @@ gs_shell_show_search (GsShell *shell, const gchar *search)
 	gs_shell_change_mode (shell, GS_SHELL_MODE_SEARCH, NULL, NULL, TRUE);
 }
 
+void
+gs_shell_show_filename (GsShell *shell, const gchar *filename)
+{
+	save_back_entry (shell);
+	gs_shell_change_mode (shell, GS_SHELL_MODE_DETAILS, NULL, (gpointer) filename, TRUE);
+	gs_shell_activate (shell);
+}
+
 typedef struct {
 	GsShell *shell;
 	GsApp *app;
