@@ -79,6 +79,8 @@ void		 gs_shell_show_category		(GsShell	*shell,
 						 GsCategory	*category);
 void		 gs_shell_show_search		(GsShell	*shell,
 						 const gchar	*search);
+void		 gs_shell_show_filename		(GsShell	*shell,
+						 const gchar	*filename);
 void		 gs_shell_show_search_result	(GsShell	*shell,
 						 const gchar	*id,
 						 const gchar    *search);
