@@ -287,12 +287,15 @@ gs_plugin_initialize (GsPlugin *plugin)
 }
 
 /**
- * gs_plugin_get_priority:
+ * gs_plugin_get_deps:
  */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
+const gchar **
+gs_plugin_get_deps (GsPlugin *plugin)
 {
-	return 1.0f;
+	static const gchar *deps[] = {
+		"datadir-apps",		/* set the state using the installed file */
+		NULL };
+	return deps;
 }
 
 /**
@@ -528,12 +531,18 @@ gs_plugin_refine_item (GsPlugin *plugin,
 		gs_app_set_id (app, appstream_app_get_id (item));
 
 	/* set name */
-	if (appstream_app_get_name (item) != NULL && gs_app_get_name (app) == NULL)
-		gs_app_set_name (app, appstream_app_get_name (item));
+	if (appstream_app_get_name (item) != NULL) {
+		gs_app_set_name (app,
+				 GS_APP_QUALITY_HIGHEST,
+				 appstream_app_get_name (item));
+	}
 
 	/* set summary */
-	if (appstream_app_get_summary (item) != NULL && gs_app_get_summary (app) == NULL)
-		gs_app_set_summary (app, appstream_app_get_summary (item));
+	if (appstream_app_get_summary (item) != NULL) {
+		gs_app_set_summary (app,
+				    GS_APP_QUALITY_HIGHEST,
+				    appstream_app_get_summary (item));
+	}
 
 	/* add urls */
 	urls = appstream_app_get_urls (item);
@@ -560,8 +569,11 @@ gs_plugin_refine_item (GsPlugin *plugin,
 		gs_app_set_keywords (app, appstream_app_get_keywords (item));
 
 	/* set description */
-	if (appstream_app_get_description (item) != NULL && gs_app_get_description (app) == NULL)
-		gs_app_set_description (app, appstream_app_get_description (item));
+	if (appstream_app_get_description (item) != NULL) {
+		gs_app_set_description (app,
+					GS_APP_QUALITY_HIGHEST,
+					appstream_app_get_description (item));
+	}
 
 	/* set icon */
 	if (appstream_app_get_icon (item) != NULL && gs_app_get_pixbuf (app) == NULL)
@@ -665,7 +677,7 @@ out:
  */
 gboolean
 gs_plugin_refine (GsPlugin *plugin,
-		  GList *list,
+		  GList **list,
 		  GsPluginRefineFlags flags,
 		  GCancellable *cancellable,
 		  GError **error)
@@ -683,7 +695,7 @@ gs_plugin_refine (GsPlugin *plugin,
 	}
 
 	gs_profile_start (plugin->profile, "appstream::refine");
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		ret = gs_plugin_refine_from_id (plugin, app, error);
 		if (!ret) {
@@ -704,6 +716,54 @@ out:
 	return ret;
 }
 
+/**
+ * gs_plugin_add_popular:
+ */
+gboolean
+gs_plugin_add_popular (GsPlugin *plugin,
+		       GList **list,
+		       GCancellable *cancellable,
+		       GError **error)
+{
+	AppstreamApp *item;
+	GPtrArray *array;
+	GsApp *app;
+	gboolean ret = TRUE;
+	guint i;
+
+	/* load XML files */
+	if (g_once_init_enter (&plugin->priv->done_init)) {
+		ret = gs_plugin_startup (plugin, cancellable, error);
+		g_once_init_leave (&plugin->priv->done_init, TRUE);
+		if (!ret)
+			goto out;
+	}
+
+	/* just add the apps without refining */
+	gs_profile_start (plugin->profile, "appstream::add-popular");
+	array = appstream_cache_get_items (plugin->priv->cache);
+	for (i = 0; i < array->len; i++) {
+		item = g_ptr_array_index (array, i);
+		if (appstream_app_get_id (item) == NULL)
+			continue;
+
+		/* require long description */
+		if (appstream_app_get_description (item) == NULL)
+			continue;
+
+		/* require at least one screenshot */
+		if (appstream_app_get_screenshots (item)->len == 0)
+			continue;
+
+		app = gs_app_new (appstream_app_get_id (item));
+		gs_plugin_add_app (list, app);
+		g_object_unref (app);
+	}
+	gs_profile_stop (plugin->profile, "appstream::add-popular");
+out:
+	return ret;
+}
+
 /**
  * gs_plugin_add_category_apps:
  */
