@@ -79,12 +79,16 @@ gs_plugin_initialize (GsPlugin *plugin)
 }
 
 /**
- * gs_plugin_get_priority:
+ * gs_plugin_get_deps:
  */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
+const gchar **
+gs_plugin_get_deps (GsPlugin *plugin)
 {
-	return 1.0f;
+	static const gchar *deps[] = {
+		"datadir-filename",	/* requires DataDir::desktop-filename */
+		"datadir-filename-local",	/* ^^^ */
+		NULL };
+	return deps;
 }
 
 /**
@@ -107,11 +111,11 @@ gs_plugin_datadir_apps_set_from_cache_item (GsApp *app,
 	gs_app_set_id (app, cache_item->id);
 	gs_app_set_metadata (app, "DataDir::desktop-icon", cache_item->icon_name);
 	if (cache_item->name != NULL)
-		gs_app_set_name (app, cache_item->name);
+		gs_app_set_name (app, GS_APP_QUALITY_HIGHEST, cache_item->name);
 	if (cache_item->pkgname != NULL)
 		gs_app_add_source (app, cache_item->pkgname);
 	if (cache_item->summary != NULL)
-		gs_app_set_summary (app, cache_item->summary);
+		gs_app_set_summary (app, GS_APP_QUALITY_HIGHEST, cache_item->summary);
 	if (cache_item->pixbuf != NULL)
 		gs_app_set_pixbuf (app, cache_item->pixbuf);
 
@@ -292,7 +296,7 @@ out:
  */
 gboolean
 gs_plugin_refine (GsPlugin *plugin,
-		  GList *list,
+		  GList **list,
 		  GsPluginRefineFlags flags,
 		  GCancellable *cancellable,
 		  GError **error)
@@ -302,7 +306,7 @@ gs_plugin_refine (GsPlugin *plugin,
 	GsApp *app;
 	gboolean ret;
 
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		ret = gs_plugin_refine_app (plugin,
 					    app,
