@@ -66,15 +66,6 @@ gs_plugin_initialize (GsPlugin *plugin)
 	g_free (epiphany);
 }
 
-/**
- * gs_plugin_get_priority:
- */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
-{
-	return 1.0f;
-}
-
 /**
  * gs_plugin_destroy:
  */
@@ -154,8 +145,8 @@ gs_plugin_add_installed_file (GsPlugin *plugin,
 
 	/* create application */
 	*app = gs_app_new (filename);
-	gs_app_set_name (*app, name);
-	gs_app_set_summary (*app, comment);
+	gs_app_set_name (*app, GS_APP_QUALITY_NORMAL, name);
+	gs_app_set_summary (*app, GS_APP_QUALITY_NORMAL, comment);
 	/* TRANSLATORS: this is the licence of the web-app */
 	gs_app_set_licence (*app, _("Proprietary"));
 	gs_app_set_state (*app, no_display ? GS_APP_STATE_AVAILABLE :
@@ -689,7 +680,7 @@ out:
  */
 gboolean
 gs_plugin_refine (GsPlugin *plugin,
-		  GList *list,
+		  GList **list,
 		  GsPluginRefineFlags flags,
 		  GCancellable *cancellable,
 		  GError **error)
@@ -707,7 +698,7 @@ gs_plugin_refine (GsPlugin *plugin,
 			goto out;
 	}
 
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		if (gs_app_get_id_kind (app) != GS_APP_ID_KIND_WEBAPP)
 			continue;
