@@ -58,12 +58,16 @@ gs_plugin_initialize (GsPlugin *plugin)
 }
 
 /**
- * gs_plugin_get_priority:
+ * gs_plugin_get_deps:
  */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
+const gchar **
+gs_plugin_get_deps (GsPlugin *plugin)
 {
-	return 150.0f;
+	static const gchar *deps[] = {
+		"appstream",		/* need pkgname */
+		"packagekit",		/* need package_id */
+		NULL };
+	return deps;
 }
 
 /**
@@ -147,8 +151,12 @@ gs_plugin_packagekit_resolve_packages_app (GPtrArray *packages,
 				if (gs_app_get_version (app) == NULL)
 					gs_app_set_version (app,
 						pk_package_get_version (package));
-				gs_app_set_metadata (app, "PackageKit::Summary",
-						     pk_package_get_summary (package));
+				gs_app_set_name (app,
+						 GS_APP_QUALITY_LOWEST,
+						 pk_package_get_name (package));
+				gs_app_set_summary (app,
+						    GS_APP_QUALITY_LOWEST,
+						    pk_package_get_summary (package));
 			}
 		}
 	}
@@ -455,11 +463,11 @@ gs_plugin_packagekit_refine_details_app (GsPlugin *plugin,
 						pk_details_get_url (details));
 			}
 			size += pk_details_get_size (details);
-			if (gs_app_get_metadata_item (app, "PackageKit::Description") == NULL) {
-				desc = gs_pk_format_desc (pk_details_get_description (details));
-				gs_app_set_metadata (app, "PackageKit::Description", desc);
-				g_free (desc);
-			}
+			desc = gs_pk_format_desc (pk_details_get_description (details));
+			gs_app_set_description (app,
+						GS_APP_QUALITY_LOWEST,
+						desc);
+			g_free (desc);
 			break;
 		}
 	}
@@ -624,7 +632,7 @@ gs_plugin_refine_requires_package_id (GsApp *app, GsPluginRefineFlags flags)
  */
 gboolean
 gs_plugin_refine (GsPlugin *plugin,
-		  GList *list,
+		  GList **list,
 		  GsPluginRefineFlags flags,
 		  GCancellable *cancellable,
 		  GError **error)
@@ -639,7 +647,7 @@ gs_plugin_refine (GsPlugin *plugin,
 
 	/* can we resolve in one go? */
 	gs_profile_start (plugin->profile, "packagekit-refine[name->id]");
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		if (gs_app_get_source_id_default (app) != NULL)
 			continue;
@@ -666,7 +674,7 @@ gs_plugin_refine (GsPlugin *plugin,
 
 	/* set the package-id for an installed desktop file */
 	gs_profile_start (plugin->profile, "packagekit-refine[desktop-filename->id]");
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION) == 0)
 			continue;
 		app = GS_APP (l->data);
@@ -687,7 +695,7 @@ gs_plugin_refine (GsPlugin *plugin,
 
 	/* any update details missing? */
 	gs_profile_start (plugin->profile, "packagekit-refine[id->update-details]");
-	for (l = list; l != NULL; l = l->next) {
+	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		if (gs_app_get_state (app) != GS_APP_STATE_UPDATABLE)
 			continue;
@@ -710,7 +718,7 @@ gs_plugin_refine (GsPlugin *plugin,
 	    (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE) > 0 ||
 	    (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION) > 0) {
 		ret = gs_plugin_refine_require_details (plugin,
-							list,
+							*list,
 							cancellable,
 							error);
 		if (!ret)
