@@ -56,15 +56,6 @@ gs_plugin_initialize (GsPlugin *plugin)
 	pk_client_set_cache_age (PK_CLIENT (plugin->priv->task), G_MAXUINT);
 }
 
-/**
- * gs_plugin_get_priority:
- */
-gdouble
-gs_plugin_get_priority (GsPlugin *plugin)
-{
-	return 10.0f;
-}
-
 /**
  * gs_plugin_destroy:
  */
